import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * Eine grafische Ansicht des Simulationsfeldes.
 * 
 * @author Ronald Ortner
 * @version 2015.01.16
 */
public class Simulationsansicht extends JFrame 
{
    private static final long serialVersionUID = 20060330L;
    
    // Die Farbe fuer tote Zellen
    private static final Color FARBE0 = Color.white;
    // Die Farbe fuer lebendige Zellen
    private static final Color FARBE1 = Color.blue;

    private JLabel infoLabel;
    private Weltansicht weltansicht;
    private WeltStatistik stats;

    /**
     * Erzeuge eine Ansicht mit der gegebenen Breite und Laenge.
     * @param laenge Die Laenge der Simulation.
     * @param breite Die Breite der Simulation.
     */
    public Simulationsansicht(int laenge, int breite)
    {
        stats = new WeltStatistik();

        setTitle("Game of Life");
        
        setLocation(100, 50);
        
        weltansicht = new Weltansicht(laenge, breite);

        infoLabel = new JLabel("", JLabel.CENTER);
        
        Container inhalt = getContentPane();
        
        inhalt.add(infoLabel, BorderLayout.NORTH);
        inhalt.add(weltansicht, BorderLayout.CENTER);
    }
    
    
    /**
     * Zeige den aktuellen Zustand der Welt.
     * @param schritt welcher Iterationsschritt ist dies.
     * @param welt die Welt, die angezeigt werden soll.
     */
    public void zeigeStatus(int schritt, Welt welt)
    {
        if(!isVisible())
            setVisible(true);

        stats.zuruecksetzen();
        
        weltansicht.zeichnenVorbereiten();
            
        for(int zeile = 0; zeile < welt.gibLaenge(); zeile++) {
            for(int spalte = 0; spalte < welt.gibBreite(); spalte++) {
                int i = welt.gibWelt(zeile, spalte);
                if(i == 1) {
                    stats.erhoeheZaehler();
                    weltansicht.zeichneMarkierung(spalte, zeile, FARBE1);
                }
                else {
                    weltansicht.zeichneMarkierung(spalte, zeile, FARBE0);
                }
            }
        }
        stats.zaehlungBeendet();

        infoLabel.setText("Schritte: " + schritt + "       Population " + stats.gibZaehler(welt));
        weltansicht.repaint();
    }

    /**
     * Entscheide, ob die Simulation weiterlaufen soll.
     * @return true wenn noch mehr als eine Spezies lebendig ist.
     */
    public boolean istAktiv(Welt welt)
    {
        return stats.istAktiv();
    }
    
    /**
     * Liefere eine grafische Ansicht eines rechteckigen Feldes.
     * Dies ist eine geschachtelte Klasse (eine Klasse, die
     * innerhalb einer anderen Klasse definiert ist), die eine
     * eigene grafische Komponente fuer die Benutzungsschnittstelle
     * definiert. Diese Komponente zeigt das Feld an.
     * Dies ist fortgeschrittene GUI-Technik - Sie koennen sie
     * fuer Ihr Projekt ignorieren, wenn Sie wollen.
     */
    private class Weltansicht extends JPanel
    {
        private static final long serialVersionUID = 20060330L;
        private final int DEHN_FAKTOR = 10;

        private int weltBreite, weltLaenge;
        private int xFaktor, yFaktor;
        Dimension groesse;
        private Graphics g;
        private Image weltImage;

        /**
         * Erzeuge eine neue Komponente zur Weltansicht.
         */
        public Weltansicht(int laenge, int breite)
        {
            weltLaenge = laenge;
            weltBreite = breite;
            groesse = new Dimension(0, 0);
        }

        /**
         * Der GUI-Verwaltung mitteilen, wie gross wir sein wollen.
         * Der Name der Methode ist durch die GUI-Verwaltung festgelegt.
         */
        public Dimension getPreferredSize()
        {
            return new Dimension(weltBreite * DEHN_FAKTOR,
                                 weltLaenge * DEHN_FAKTOR);
        }
        
        /**
         * Bereite eine neue Zeichenrunde vor. Da die Komponente
         * in der Groesse geaendert werden kann, muss der Massstab neu
         * berechnet werden.
         */
        public void zeichnenVorbereiten()
        {
            if(! groesse.equals(getSize())) {  // Groesse wurde geaendert...
                groesse = getSize();
                weltImage = weltansicht.createImage(groesse.width, groesse.height);
                g = weltImage.getGraphics();

                xFaktor = groesse.width / weltBreite;
                if(xFaktor < 1) {
                    xFaktor = DEHN_FAKTOR;
                }
                yFaktor = groesse.height / weltLaenge;
                if(yFaktor < 1) {
                    yFaktor = DEHN_FAKTOR;
                }
            }
        }
        
        /**
         * Zeichne an der gegebenen Position ein Rechteck mit
         * der gegebenen Farbe.
         */
        public void zeichneMarkierung(int x, int y, Color farbe)
        {
            g.setColor(farbe);
            g.fillRect(x * xFaktor, y * yFaktor, xFaktor-1, yFaktor-1);
        }

        /**
         * Die Komponente fuer die Feldansicht muss erneut angezeigt
         * werden. Kopiere das interne Image in die Anzeige.
         * Der Name der Methode ist durch die GUI-Verwaltung festgelegt.
         */
        public void paintComponent(Graphics g)
        {
            if(weltImage != null) {
                Dimension aktuelleGroesse = getSize();
                if(groesse.equals(aktuelleGroesse)) {
                	g.drawImage(weltImage, 0, 0, null);
                }
                else {
                    // Groesse des aktuellen Images anpassen.
                    g.drawImage(weltImage, 0, 0, aktuelleGroesse.width,
                    		    aktuelleGroesse.height, null);
                }
            }
        }
    }
}
